=== modified file 'NEWS'
--- NEWS	2007-02-01 17:18:42 +0000
+++ NEWS	2007-02-01 20:24:03 +0000
@@ -36,6 +36,9 @@
 
   BUGFIXES:
 
+    * Lock operations no longer hang if they encounter a permission problem.
+      (Aaron Bentley)
+
   TESTING:
 
     * New ``--first`` option to ``bzr selftest`` to run specified tests

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2006-11-10 21:06:11 +0000
+++ bzrlib/lockdir.py	2007-01-31 18:49:12 +0000
@@ -220,6 +220,8 @@
             self.transport.rename(tmpname, self._held_dir)
             self._lock_held = True
             self.confirm()
+        except errors.PermissionDenied:
+            raise
         except (PathError, DirectoryNotEmpty, FileExists, ResourceBusy), e:
             mutter("contention on %r: %s", self, e)
             raise LockContention(self)

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2007-01-04 23:36:44 +0000
+++ bzrlib/osutils.py	2007-02-01 14:59:04 +0000
@@ -915,6 +915,19 @@
     return sys.platform != "win32"
 
 
+def supports_posix_readonly():
+    """Return True if 'readonly' has POSIX semantics, False otherwise.
+
+    Notably, a win32 readonly file cannot be deleted, unlike POSIX where the
+    directory controls creation/deletion, etc.
+
+    And under win32, readonly means that the directory itself cannot be
+    deleted.  The contents of a readonly directory can be changed, unlike POSIX
+    where files in readonly directories cannot be added, deleted or renamed.
+    """
+    return sys.platform != "win32"
+
+
 def set_or_unset_env(env_variable, value):
     """Modify the environment, setting or removing the env_variable.
 

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2006-09-29 20:12:58 +0000
+++ bzrlib/tests/test_lockdir.py	2007-02-01 14:59:04 +0000
@@ -17,13 +17,16 @@
 """Tests for LockDir"""
 
 from cStringIO import StringIO
+import os
 from threading import Thread, Lock
 import time
 
 import bzrlib
 from bzrlib import (
     config,
+    errors,
     osutils,
+    tests,
     )
 from bzrlib.errors import (
         LockBreakMismatch,
@@ -600,3 +603,12 @@
         ld1.create()
         ld1.lock_write()
         ld1.unlock()
+
+    def test_lock_permission(self):
+        if not osutils.supports_posix_readonly():
+            raise tests.TestSkipped('Cannot induce a permission failure')
+        ld1 = self.get_lock()
+        lock_path = ld1.transport.local_abspath('test_lock')
+        os.mkdir(lock_path)
+        osutils.make_readonly(lock_path)
+        self.assertRaises(errors.PermissionDenied, ld1.attempt_lock)

